use crate::{types::CasperExtrinsicDetails, action::Action, casper::CasperBlock};
use color_eyre::Result;

pub struct FinalizedSubscription {
    action_tx: tokio::sync::mpsc::UnboundedSender<Action>, 
    network_tx: std::sync::mpsc::Sender<Action>,
    finalized_blocks_sub: subxt::backend::StreamOfResults<CasperBlock>,
}

impl FinalizedSubscription {
    pub fn new(
        action_tx: tokio::sync::mpsc::UnboundedSender<Action>, 
        network_tx: std::sync::mpsc::Sender<Action>,
        finalized_blocks_sub: subxt::backend::StreamOfResults<CasperBlock>,
    ) -> Self {
        Self { action_tx, network_tx, finalized_blocks_sub }
    }

    pub async fn subscribe_finalized_blocks(&mut self) -> Result<()> {
        while let Some(block) = self.finalized_blocks_sub.next().await {
            let block = block?;
            let block_number = block.header().number;
            let block_hash = block.hash();
            let extrinsics = block.extrinsics().await?;

            let mut extrinsic_details = Vec::new();
            for ext in extrinsics.iter() {
                let pallet_name = match ext.pallet_name() {
                    Ok(pallet_name) => pallet_name,
                    Err(_) => continue,
                };

                let variant_name = match ext.variant_name() {
                    Ok(variant_name) => variant_name,
                    Err(_) => continue,
                };

                extrinsic_details.push(CasperExtrinsicDetails::new(
                    ext.index(),
                    ext.hash(),
                    ext.is_signed(),
                    ext.field_bytes().to_vec(),
                    ext.address_bytes().map(|addr| addr.to_vec()),
                    pallet_name.to_string(),
                    variant_name.to_string(),
                ));
            }

            self.action_tx.send(Action::FinalizedBlockInformation(block_hash, block_number))?;
            self.action_tx.send(Action::ExtrinsicsForBlock(block_number, extrinsic_details))?;
            self.action_tx.send(Action::NewFinalizedBlock(block_number))?;

            self.network_tx.send(Action::NewFinalizedHash(block_hash))?;
            self.network_tx.send(Action::GetBlockAuthor(block_hash, block.header().digest.logs.clone()))?;
        }
        Ok(())
    }
}

pub struct BestSubscription {
    action_tx: tokio::sync::mpsc::UnboundedSender<Action>, 
    network_tx: std::sync::mpsc::Sender<Action>,
    best_blocks_sub: subxt::backend::StreamOfResults<CasperBlock>,
}

impl BestSubscription {
    pub fn new(
        action_tx: tokio::sync::mpsc::UnboundedSender<Action>, 
        network_tx: std::sync::mpsc::Sender<Action>,
        best_blocks_sub: subxt::backend::StreamOfResults<CasperBlock>,
    ) -> Self {
        Self { action_tx, network_tx, best_blocks_sub }
    }

    pub async fn subscribe_best_blocks(&mut self) -> Result<()> {
        while let Some(block) = self.best_blocks_sub.next().await {
            let block = block?;
            let block_number = block.header().number;
            let block_hash = block.hash();
            let extrinsics = block.extrinsics().await?;
            let extrinsics_length = extrinsics.len();

            let mut extrinsic_details = Vec::new();
            for ext in extrinsics.iter() {
                let pallet_name = match ext.pallet_name() {
                    Ok(pallet_name) => pallet_name,
                    Err(_) => continue,
                };

                let variant_name = match ext.variant_name() {
                    Ok(variant_name) => variant_name,
                    Err(_) => continue,
                };

                extrinsic_details.push(CasperExtrinsicDetails::new(
                    ext.index(),
                    ext.hash(),
                    ext.is_signed(),
                    ext.field_bytes().to_vec(),
                    ext.address_bytes().map(|addr| addr.to_vec()),
                    pallet_name.to_string(),
                    variant_name.to_string(),
                ));
            }

            self.action_tx.send(Action::BestBlockInformation(block_hash, block_number))?;
            self.action_tx.send(Action::ExtrinsicsForBlock(block_number, extrinsic_details))?;
            self.action_tx.send(Action::BestBlockUpdated(block_number))?;
            self.action_tx.send(Action::NewBestBlock(block_number))?;
            self.action_tx.send(Action::ExtrinsicsLength(block_number, extrinsics_length))?;

            self.network_tx.send(Action::NewBestHash(block_hash))?;
            self.network_tx.send(Action::GetBlockAuthor(block_hash, block.header().digest.logs.clone()))?;
            self.network_tx.send(Action::GetActiveEra)?;
            self.network_tx.send(Action::GetEpochProgress)?;
            self.network_tx.send(Action::GetTotalIssuance)?;
        }
        Ok(())
    }
}
